from framex.utils._colors import (
    black,
    blue,
    bold,
    cyan,
    green,
    magenta,
    red,
    white,
    yellow,
)

__all__ = [
    "black",
    "blue",
    "cyan",
    "green",
    "magenta",
    "red",
    "white",
    "yellow",
    "bold",
]
